
#===============================================================================
# ProfileTool - Windows 7 profile utility
# 
# Copies, deletes, purges and sets profiles as default
#
# (c) 2010 by Toussaint OTTAVI (t.ottavi@medi.fr)
#===============================================================================

#===============================================================================
#    ProfileTool is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#===============================================================================

import os, sys, time

#===============================================================================
# Help & information procedures
#===============================================================================

Version = "v0.9.3"

#--------------------------------------------------------- Displays startup text

def PrintStart():
  print
  print "  ProfileTool " + Version + " - (c) 2010 by Toussaint OTTAVI (t.ottavi@medi.fr)" 
  print "  This is free software. This program comes with ABSOLUTELY NO WARRANTY."
  print 


#------------------------------------------------------------ Displays Help text

def HaltUsageText():
  UsageText = """Usage : ProfileTool <Command> <ProfileName> [DestProfileName] [Options]

  ProfileTool is a command-line utility for handling various operations with
  user profiles under Windows 7. Some of these operations may not be considered
  as good practices by Microsoft. Ensure you know what you're doing, and use it
  at your own risk. See README for more information.  
   
    Valid commands are : 
      LIST       : Lists all the available profiles in the Users directory 
      GETSIZE    : Calculates size of <ProfileName>
      GETSIZEALL : Calculates sizes of all profiles
      DELTEMP    : Deletes temporary files from <ProfileName> 
      DELTEMPALL : Deletes temporary files from all profiles (except current) 
      DELETE     : Deletes the <ProfileName> from Users directory 
      COPY       : Copies <ProfileName> over <DestProfileName>  
      SETDEFAULT : Sets <ProfileName> as default profile 
      
    Valid options are : 
      -H : Displays this help text
      -L : Displays licensing information
      -V : Displays additional (verbose) information during process 
      -D : With COPY or SETDEFAULT, also copies Documents, Pictures, Music, etc
      -T : Test mode (simulation), won't copy or delete any file """
  print UsageText   
  sys.exit()

#------------------------------------------------------- Displays Licensing text

def HaltLicense():
  print """  ProfileTool is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  """
  print  "Type ProfileTool -h for Help"
  sys.exit()
  
#---------------------------------------------------- Various display procedures

def HaltHelp():
  print  "Type ProfileTool -h for Help, ProfileTool -l for Licensing information"
  sys.exit()
  
def HaltErrorHelp(message):
  print 'ERROR ' + message
  HaltHelp()

def HaltError(message):
  print 'ERROR : ' + message
  sys.exit()
  
def PrintDebug (text):
  if FlagDebug:
    print('  DEBUG: '+ text)

#===============================================================================
# Rotating progress indicator
#===============================================================================

class cRotatingProgress:
  
  n = 0
  LastTime = 0.0
  
#----------------------------------------------------- Init of rotating progress
  
  def __init__(self):
    self.n = 0
    self.LastTime = time.clock()
    sys.stdout.write('O')
    sys.stdout.flush()

#---------- Displays rotating progress. Call as often as possible during process
  
  def Rotate(self): 
    TempTime = time.clock()
    if (TempTime - self.LastTime)>=0.1:
      self.LastTime = TempTime
      c=''
      self.n = self.n+1
      if self.n==1:
        c = '-'
      elif self.n==2:
        c = '\\'
      elif self.n==3: 
        c = '|'
      elif self.n==4:
        c = '/'
        self.n=0
      sys.stdout.write('\b')
      sys.stdout.write(c)
      sys.stdout.flush()
  
  #------------------------------------ Clear rotating process. Call at the end.
    
  def Clear(self):  
    self.n = 0
    sys.stdout.write('\b')
    sys.stdout.flush()
    

#===============================================================================
# Copies data with Robocopy
# (does not copy junctions on directories, /SL option works only with files)
#===============================================================================

def Robocopy (source, dest):

  print "  " + source + " -> " + dest
    
  RobocopyOptions = "/e /zb /copy:DAT /np /xj /r:2 /w:1"
  QuietMode = "" if FlagDebug else "/nfl /ndl /njh /njs"
  LogMode = " /tee /log+:copyprofile.log" if FlagLog else "" 
  RobocopyLine = "robocopy %s %s " + RobocopyOptions + " " + QuietMode + " " + LogMode

  source = '"' + source + '"'
  dest = '"' + dest + '"'
   
  if FlagTest:
    if FlagDebug: 
      print "  Test Mode : " + RobocopyLine % (source, dest)
  else:
    try:
      os.system (RobocopyLine % (source, dest))
    except:
      print "  ERROR : Unable to use Robocopy command"
   

#===============================================================================
# Copies data with Xcopy
# (often fails when copying .lnk files) 
#===============================================================================

def XCopy (source, dest):
  
  print "  " + source + " -> " + dest
  
  Options = "/C /H /R /Y"
  QuietMode = "" if FlagDebug else "/Q"
  XCopyLine = "xcopy " + Options + " " + QuietMode +" %s %s " 
  
  source = '"' + source + '"'
  dest = '"' + dest + '"'
  
  if FlagTest:
    if FlagDebug:
      print "  Test Mode : " + XCopyLine % (source, dest)
  else:
    try:
      os.system ( XCopyLine % (source, dest))
    except:
      print "  ERROR: Unable to use XCopy command"


#===============================================================================
# Deletes a directory and its contents recursively without confirmation
#===============================================================================

def DeleteDir (DirName):
    
  Options = "/S" if FlagDebug else "/S /Q"
  RdLine = "rd %s " + Options 
  DirNameQuoted = '"'+DirName+'"'
  print
  if os.path.isdir (DirName):
    print "  -> Deleting directory " + DirName
    if FlagTest:
      print "  Test Mode : " + RdLine % (DirName)     
    else:
      try:
        os.system ( RdLine % (DirNameQuoted))
      except:
        print "  EXCEPTION: Unable to use RD command"
      if os.path.isdir (DirName):                 # If directory is still here, try again
        print "  -> Deleting directory " + DirName +" (Attempt #2)"
        try:
          os.system ( RdLine % (DirNameQuoted))
        except:
          print "  EXCEPTION: Unable to use RD command"
      if os.path.isdir (DirName):                 # If still here after attempt 2
        print("  -> Directory not completely deleted. Some files may be in use. Try again.")         
  else:  
    print "  -> Deleting directory " + DirName + " : <Not Found>"
 
 
#===============================================================================
# Deletes temporary files in selected profile
#===============================================================================

def DelTemp (profile, profilepath):
  
  print "Deleting temporary files in profile "+profile
  for dir in TempList:
    DeleteDir (profilepath + dir)
    
    
#===============================================================================
# Delete temporary files in all user profiles
#===============================================================================

def DelTempAll():
  
  print "Deleting temporary files in all profiles"  
  for Profile in GoodProfiles:   
    print  
    ProfilePath = UsersPath + "\\" + Profile + "\\"
    DelTemp (Profile, ProfilePath)
   
    
#===============================================================================
# Copy profile content (all selected files and directories)
#===============================================================================

def CopyProfile():

  # Copy standard subdirs
  print('  -> Copying standard profile subdirectories')
  for name in ProfileStandardList: 
    Robocopy (SourcePath + name, DestPath + name)
  
  # Copy Documents (if selected)
  if FlagDocuments:
    print
    print('  -> Copying user documents')
    for name in ProfileDocumentList: 
      Robocopy (SourcePath + name, DestPath + name) 
  
  # Files in the profile root (USER.DAT, etc..)
  print
  print('  -> Copying registry and root files')
  XCopy ( SourcePath + "*.*", DestPath)


#===============================================================================
# Get size of a folder and all its contents
#===============================================================================

def GetFolderSize (FullPath):
  try:
    rot = cRotatingProgress()
    size = 0 
    PrintDebug ('Getting size of '+FullPath)
    
    for root, dirs, files in os.walk(FullPath):
      for name in files:
        tempsize = os.path.getsize(os.path.join(root, name))
        size = size + tempsize
        rot.Rotate()
    rot.Clear()
    PrintDebug('Total size : '+str(size))
  except:
    HaltError ('Error computing directory size')  
  return size / (1024*1024)

  
#===============================================================================
# Get global variables
#===============================================================================

def GetGlobals():
  
  global UsersPath, CurrentUser, GoodProfiles
  
  PrintDebug('Getting global variables')
  # Get Users directory path  
  try:
    UserProfilePath = os.getenv('USERPROFILE')
    PrintDebug('Current user profile : '+ UserProfilePath)
    
    UsersPath, CurrentUser = os.path.split(UserProfilePath)
    CurrentUser = CurrentUser.upper()
    PrintDebug ('Users directory : '+ UsersPath)
    PrintDebug ('Current user : ' + CurrentUser)
  except:
    HaltError ('Error getting profile name and path')
  
  # Get list of existing profiles  
  try:
    PrintDebug('Browsing user profile directory')
    CurrentProfiles = os.listdir(UsersPath)
    for item in CurrentProfiles:
      temp = os.path.join(UsersPath,item)
      PrintDebug('  Checking name '+temp)
      if os.path.isdir(temp):
        if item.upper() not in ('ALL USERS', 'DEFAULT', 'DEFAULT USER', 'PUBLIC'):
          PrintDebug('    Found profile :' + item)
          GoodProfiles.append(item.upper())
  except:
    HaltError('Unable to browse user profile directory')   
    

#===============================================================================
# Display simple list of existing profiles
#===============================================================================

def ListProfiles():
  
  print ('Your current profile is : ' + CurrentUser)
  print
  print ('List of existing profiles :')
  for Profile in GoodProfiles:
    print '  ' + Profile

  
#===============================================================================
# Compute size of a profile
#===============================================================================

def GetSize(name):

  try:  
    FullPath = os.path.join(UsersPath,name)
    print ('  ' + name + ' - Documents '),
    # Size of Documents
    DocumentSize = 0
    for doc in ProfileDocumentList:
      DocumentSize = DocumentSize + GetFolderSize (os.path.join(FullPath,doc))
    print '(%d Mb), Temporary ' % DocumentSize,
    # Size of temporary files
    TempSize = 0
    for tmp in TempList:
        TempSize = TempSize + GetFolderSize (os.path.join(FullPath,tmp))     
    print '(%d Mb), Local ' % TempSize,
    # Size of AppData\Local
    LocalSize = GetFolderSize (os.path.join(FullPath,'AppData\\Local'))
    print '(%d Mb), Roaming ' % LocalSize,           
    # Size of AppData\Roaming
    RoamingSize = GetFolderSize (os.path.join(FullPath,'AppData\\Roaming'))
    print '(%d Mb), TOTAL: ' % RoamingSize,           
    # Total size
    TotalSize = GetFolderSize (FullPath)
    print '%d Mb' % TotalSize         
  except:
    HaltError ('Error listing profiles')
    
    
#===============================================================================
# Get list of all profiles with their sizes
#===============================================================================

def GetSizeAll():
  
  try:
    for name in GoodProfiles:
      GetSize(name)         
  except:
    HaltError ('Error listing profiles')


#===============================================================================
# Main program
#===============================================================================

#-------------------------------------------------------------- Global variables

CurrentUser = ''      # Name of logged-in current user
UsersPath = ''        # Path of the Users directory
GoodProfiles = []     # List of all normal user profiles in Users directory

SourceProfile = ''    # Name of source profile
DestProfile = ''      # Name of dest profile
SourcePath = ''       # Full path of source profile
DestPath = ''         # Full path of dest profile

FlagDebug = False       # Debug / Verbose flag
FlagLog = False         # TODO: Log results to a file
FlagDocuments = False   # Also copy user documents, pictures, music, etc...
FlagTest = False        # TEST mode (simulation)

#--------------------------------- List of standard profile subdirs to be copied

ProfileStandardList = ["AppData\\Roaming", 
                       "Contacts", 
                       "Desktop", 
                       "Favorites", 
                       "Links", 
                       "Searches"]

#----------------------------- List of user subdirs to be treated as "documents"

ProfileDocumentList = ["Documents", 
                       "Music", 
                       "Pictures", 
                       "Saved Games", 
                       "Videos", 
                       "Downloads"]

#-----------------------------  List of temporary files to be deleted by DELTEMP

TempList = ["AppData\\Local\\temp",
            "AppData\\Local\\Microsoft\\Windows\\Temporary Internet Files",
            "AppData\\Local\\Microsoft\\Windows\\History",
            "AppData\\Local\\Mozilla\\Firefox\\Profiles"]


#----------------------------------------------------------------------- Startup

PrintStart()

#------------------------------------------------- Command line argument parsing

Arguments = []
Command = ''
Parameters = []
NbParameters = 0
SkipFirst = False

NbArgs = len(sys.argv)-1
if (NbArgs<1): HaltHelp()

# Put all arguments in upper case. Warning, Arguments[0] is command line
for Arg in sys.argv: Arguments.append(Arg.upper()) 
    
# Parsing arguments
for Arg in Arguments: 
  if SkipFirst == False:        # Skipping first argument (command line)
    SkipFirst = True            
  else:                         # Checking for valid flags
    if Arg in ("-H", "-HELP", "-?"):        
      HaltUsageText()
    elif Arg == "-L":
      HaltLicense()  
    elif Arg == "-V":                       
      FlagDebug = True
      PrintDebug('Activating Verbose / Debug mode')
    elif Arg == "-D":
      FlagDocuments = True
      PrintDebug('Will also copy Documents')
    elif Arg == "-T":
      FlagTest = True
      PrintDebug("Test mode (simulation): won't copy or delete anything")
    else:
      if Command == '':          # First non-flag argument is a command
        Command = Arg
      else:                      # following non-flags are parameters      
        Parameters.append(Arg)

#----------------------------------------------------------- Processing Commands

if Command == '': HaltError ('Syntax Error : You must specify a command')

NbParameters = len(Parameters)

# ====== List of existing profiles
if  Command == "LIST": 
  GetGlobals()
  ListProfiles() 

# ====== Size of a profile
elif Command == "GETSIZE": 
  if NbParameters<>1:
    HaltError("GETSIZE needs 1 parameter (profile name)") 
  SourceProfile = Parameters [0] 
  GetGlobals()
  if SourceProfile not in GoodProfiles : 
    HaltError (SourceProfile + ' is not a valid profile name. Use LIST command.')
  print ('Size of user profile :')
  GetSize(SourceProfile)

# ====== Size of all profiles
elif Command == "GETSIZEALL": 
  GetGlobals()
  print ('Your current profile is : ' + CurrentUser)
  print
  print ('Size of all user profiles :')
  GetSizeAll()

# ====== Deletes selected profile
elif Command == "DELETE":                             
  if NbParameters <> 1:
    HaltError("DELETE needs 1 parameter (profile name)") 
  SourceProfile = Parameters [0]   
  GetGlobals()
  if (SourceProfile == CurrentUser):
    HaltError('You can not delete your current profile (' + CurrentUser +')')    
  if SourceProfile in GoodProfiles : 
    SourcePath = UsersPath + "\\" + SourceProfile + "\\"
  else:
    HaltError (SourceProfile + ' is not a valid profile name. Use LIST command.')  
  DeleteDir(SourcePath)

# ====== Delete temporary files from selected profile
elif Command == "DELTEMP":                              
  if NbParameters <> 1:
    HaltError("DELTEMP needs 1 parameter (profile name)") 
  SourceProfile = Parameters [0]   
  GetGlobals()
  if SourceProfile in GoodProfiles : 
    SourcePath = UsersPath + "\\" + SourceProfile + "\\"
  else:
    HaltError (SourceProfile + ' is not a valid profile name. Use LIST command.')  
  DelTemp(SourceProfile, SourcePath)

# ====== PurgeAll selected profile
elif Command == "DELTEMPALL":                          
  if NbParameters <> 0:
    HaltError("DELTEMPALL needs no parameter")   
  GetGlobals()
  DelTempAll()
  
# ====== Copy a profile over another one
elif Command == "COPY":   
  if NbParameters<>2:
    HaltError("COPY needs 2 parameters (source and destination profile names") 
  SourceProfile = Parameters [0]
  DestProfile = Parameters[1]    
  GetGlobals()
  if (SourceProfile == CurrentUser) or (DestProfile == CurrentUser):
    HaltError('You are logged in as ' + CurrentUser +'. It can not be source or destination of a copy')    
  if SourceProfile == DestProfile : HaltError ('Can not copy '+SourceProfile +' over itself') 
  if SourceProfile in GoodProfiles : 
    SourcePath = UsersPath + "\\" + SourceProfile + "\\"
  else:
    HaltError (SourceProfile + ' is not a valid source profile name. Use LIST command.')  
  if DestProfile in GoodProfiles: 
    DestPath = UsersPath + "\\" + DestProfile + "\\"
  else:
    HaltError (DestProfile + ' is not a valid destination profile name. Use LIST command.') 
  CopyProfile()

# ====== Copy selected profile over default profile   
elif Command == "SETDEFAULT":                           
  if NbParameters <> 1:
    HaltError("SETDEFAULT needs 1 parameter (source profile name)") 
  SourceProfile = Parameters [0]
  DestProfile = 'DEFAULT'    
  GetGlobals()
  if (SourceProfile == CurrentUser):
    HaltError('You are logged in as ' + CurrentUser +'. It can not be set as default because it is in use')    
  if SourceProfile in GoodProfiles : 
    SourcePath = UsersPath + "\\" + SourceProfile + "\\"
  else:
    HaltError (SourceProfile + ' is not a valid source profile name. Use LIST command.')  
  DestPath = UsersPath + "\\" + DestProfile + "\\"
  CopyProfile()  

# ====== Unrecognized command
else: 
  HaltErrorHelp("Syntax Error : Unrecognized command " + Command)  


#--------------------------------------------------------------------------- End

print
print "Done"
